@@ -1,6 +1,5 @@
-from .plot import plot, AnimatedAccessor
-from .base import load_dataset, animate_multiple_plots
-from .config import output_file, output_html
+from .plot import plot, AnimatedAccessor, animate_multiple_plots
+from .base import load_dataset
 
 # Register animated_plot accessor for Pandas DataFrames and Series:
 import pandas as pd
