@@ -44,7 +44,7 @@ DARK24 = [
 #             anim.save(chart.filename, fps=chart.fps)
 
 
-@attr.s
+@attr.s()
 class BaseChart:
     df: pd.DataFrame = attr.ib()
     use_index: bool = attr.ib()
@@ -56,12 +56,23 @@ class BaseChart:
     cmap: Union[str, colors.Colormap, List[str]] = attr.ib()
     kwargs = attr.ib()
 
-    def validate_params(self) -> None:
+    @fig.validator
+    def validate_params(self, attribute, value) -> None:
         if self.fig is not None and not isinstance(self.fig, plt.Figure):
             raise TypeError("`fig` must be a matplotlib Figure instance")
 
+    def init_func(self):
+        raise NotImplementedError("Initializing method not yet implemented")
+
     def anim_func(self, frame):
-        raise NotImplementedError("Animation function not yet implemented")
+        raise NotImplementedError("Animation method not yet implemented")
+
+    def get_frames(self):
+        raise NotImplementedError("Get frames method not yet implemented")
+
+    # def preview(self,number_of_frames=5):
+    #     anim = self.make_animation(number_of_frames, self.init_func)
+    #     plt.show()
 
     def make_animation(self, frames, init_func) -> FuncAnimation:
 
@@ -70,6 +81,70 @@ class BaseChart:
             self.fig, self.anim_func, frames, init_func, interval=interval,
         )
 
+    ## TODO Implement standard figure layouts
+
+    # def create_figure(self):
+    #     fig = plt.Figure(figsize=self.figsize, dpi=self.dpi)
+    #     limit = (0.2, self.n_bars + 0.8)
+    #     rect = self.calculate_new_figsize(fig)
+    #     ax = fig.add_axes(rect)
+    #     if self.orientation == "h":
+    #         ax.set_ylim(limit)
+    #         ax.grid(True, axis="x", color="white")
+    #         ax.xaxis.set_major_formatter(ticker.StrMethodFormatter("{x:,.0f}"))
+    #     else:
+    #         ax.set_xlim(limit)
+    #         ax.grid(True, axis="y", color="white")
+    #         ax.set_xticklabels(ax.get_xticklabels(), ha="right", rotation=30)
+    #         ax.yaxis.set_major_formatter(ticker.StrMethodFormatter("{x:,.0f}"))
+
+    #     ax.set_axisbelow(True)
+    #     ax.tick_params(length=0, labelsize=self.tick_label_size, pad=2)
+    #     ax.set_facecolor(".9")
+    #     ax.set_title(self.title)
+    #     for spine in ax.spines.values():
+    #         spine.set_visible(False)
+    #     return fig, ax
+
+
+    # def show_period(self,i):
+    #     if self.use_index:
+    #         self.orig_index = self.df.index.astype("str")
+    #         val = self.orig_index[i // self.steps_per_period]
+    #         if self.append_period_to_title:
+    #             self.ax.set_title(
+    #                 f"{'' if self.title is None else self.title}{' : ' if self.title is not None else ''}{val}"
+    #             )
+    #         else:
+    #             num_texts = len(self.ax.texts)
+    #             if num_texts == 0:
+    #                 self.ax.text(
+    #                     self.x_label,
+    #                     self.y_label,
+    #                     val,
+    #                     transform=self.ax.transAxes,
+    #                     fontsize=self.period_label_size,
+    #                 )
+    #             else:
+    #                 self.ax.texts[0].set_text(val)
+
+    def save(self, filename):
+        # Inspiration for design pattern https://github.com/altair-viz/altair/blob/c55707730935159e4e2d2c789a6dd2bc3f1ec0f2/altair/utils/save.py
+        # https://altair-viz.github.io/user_guide/saving_charts.html
+
+        anim = self.make_animation(self.get_frames(), self.init_func)
+        self.fps = 1000 / self.period_length * self.steps_per_period
+
+        extension = filename.split(".")[-1]
+        if extension == "gif":
+            anim.save(filename, fps=self.fps, writer="imagemagick")
+        else:
+            anim.save(filename, fps=self.fps)
+
+    def get_html5_video(self):
+        anim = self.make_animation(self.get_frames(), self.init_func)
+        return anim.to_html5_video()
+
     def get_data_cols(self) -> List[str]:
         data_cols = []
         for i, col in enumerate(self.df.columns):
@@ -90,7 +165,15 @@ class BaseChart:
 
     def get_colors(self, cmap):
         if isinstance(cmap, str):
-            cmap = DARK24 if cmap == "dark24" else plt.cm.get_cmap(cmap)
+            try:
+                cmap = DARK24 if cmap == "dark24" else plt.cm.get_cmap(cmap)
+            except ValueError:
+                # Try setting a list of repeating colours if no cmap found (for single colours)
+                cmap = [colors.to_rgba(cmap)] * len(self.get_data_cols())
+            except:
+                raise ValueError(
+                    "Provide a suitable color name or color map as per matplotlib"
+                )
         if isinstance(cmap, colors.Colormap):
             chart_colors = cmap(range(cmap.N)).tolist()
         elif isinstance(cmap, list):
@@ -106,7 +189,7 @@ class BaseChart:
         return chart_colors
 
 
-@attr.s
+@attr.s()
 class BarChart(BaseChart):
     orientation: str = attr.ib()
     sort: str = attr.ib()
@@ -115,17 +198,36 @@ class BarChart(BaseChart):
     bar_label_size: Union[int, float] = attr.ib()
     tick_label_size: Union[int, float] = attr.ib()
     period_label_size: Union[int, float] = attr.ib()
+    x_period_label_location: Union[int, float] = attr.ib()
+    y_period_label_location: Union[int, float] = attr.ib()
+    append_period_to_title: bool = attr.ib()
 
     def __attrs_post_init__(self):
         self.n_bars = self.n_bars or self.df.shape[1]
         self.df_values, self.df_rank = self.prepare_data()
         self.orig_index = self.df.index.astype("str")
         self.dpi = 144
-        self.fps = 1000 / self.period_length * self.steps_per_period
         if self.fig is None:
             self.fig, self.ax = self.create_figure()
+        else:
+            self.ax = self.fig.axes[0]
         self.ax.set_title(self.title)
-        self.x_label, self.y_label = self.get_label_position()
+        if self.x_period_label_location is None or self.y_period_label_location is None:
+            self.x_label, self.y_label = self.get_label_position()
+        else:
+            if self.x_period_label_location is not None:
+                self.x_label = self.x_period_label_location
+            else:
+                raise ValueError(
+                    f"Provide x_period_label_location, current value: {self.x_period_label_location}"
+                )
+            if self.y_period_label_location is not None:
+                self.y_label = self.y_period_label_location
+            else:
+                raise ValueError(
+                    f"Provide y_period_label_location, current value: {self.y_period_label_location}"
+                )
+
         self.bar_colors = self.get_colors(self.cmap)
 
     def validate_params(self):
@@ -139,6 +241,7 @@ class BarChart(BaseChart):
 
     def get_colors(self, cmap):
         bar_colors = super().get_colors(cmap)
+
         # bar_colors is now a list
         n = len(bar_colors)
         if self.df.shape[1] > n:
@@ -196,7 +299,9 @@ class BarChart(BaseChart):
         import io
 
         # df_values = self.prepare_data()
-        fig = plt.Figure(tight_layout=True, figsize=self.figsize)
+        fig = plt.Figure(figsize=self.figsize)
+        if self.title:
+            fig.tight_layout(rect=[0, 0, 1, 0.9])  # To include title
         ax = fig.add_subplot()
         fake_cols = [chr(i + 70) for i in range(self.df.shape[1])]
 
@@ -273,17 +378,22 @@ class BarChart(BaseChart):
 
         if self.use_index:
             val = self.orig_index[i // self.steps_per_period]
-            num_texts = len(self.ax.texts)
-            if num_texts == 0:
-                self.ax.text(
-                    self.x_label,
-                    self.y_label,
-                    val,
-                    transform=self.ax.transAxes,
-                    fontsize=self.period_label_size,
+            if self.append_period_to_title:
+                self.ax.set_title(
+                    f"{'' if self.title is None else self.title}{' : ' if self.title is not None else ''}{val}"
                 )
             else:
-                self.ax.texts[0].set_text(val)
+                num_texts = len(self.ax.texts)
+                if num_texts == 0:
+                    self.ax.text(
+                        self.x_label,
+                        self.y_label,
+                        val,
+                        transform=self.ax.transAxes,
+                        fontsize=self.period_label_size,
+                    )
+                else:
+                    self.ax.texts[0].set_text(val)
 
         if self.label_bars:
             for text in self.ax.texts[int(self.use_index) :]:
@@ -329,20 +439,11 @@ class BarChart(BaseChart):
     def get_frames(self):
         return range(len(self.df_values))
 
-    def make_animation(self, filename):
-
-        anim = super().make_animation(self.get_frames(), self.init_func)
-
-        extension = filename.split(".")[-1]
-        if extension == "gif":
-            anim.save(filename, fps=self.fps, writer="imagemagick")
-        else:
-            anim.save(filename, fps=self.fps)
-
 
 @attr.s
 class LineChart(BaseChart):
     line_width: int = attr.ib()
+    enable_legend: bool = attr.ib()
 
     def __attrs_post_init__(self):
         self.data_cols = self.get_data_cols()
@@ -360,7 +461,6 @@ class LineChart(BaseChart):
             self._lines[name] = {}
             self._lines[name]["x"] = []
             self._lines[name]["y"] = []
-        self.fps = 1000 / self.period_length * self.steps_per_period
         self.prepare_data()
 
     def prepare_data(self):
@@ -384,10 +484,11 @@ class LineChart(BaseChart):
         #                                           periods=len(self.df.index)))
 
     def plot_line(self, i):
+        # TODO Make ylim dynamic changing?
         self.ax.set_xlim(self.df.index[: i + 1].min(), self.df.index[: i + 1].max())
         self.ax.set_ylim(
-            self.df.select_dtypes(include=[pd.np.number]).min().min(),
-            self.df.select_dtypes(include=[pd.np.number]).max().max(),
+            self.df.iloc[:i+1].select_dtypes(include=[np.number]).min().min(skipna=True),
+            self.df.iloc[:i+1].select_dtypes(include=[np.number]).max().max(skipna=True),
         )
         for name, color in zip(self.data_cols, self.line_colors):
 
@@ -404,6 +505,8 @@ class LineChart(BaseChart):
         for line in self.ax.lines:
             line.remove()
         self.plot_line(i)
+        if self.enable_legend:
+            self.ax.legend(self.ax.lines,self._lines.keys(),**self.kwargs)
 
     def init_func(self) -> None:
         self.ax.plot([], [], self.line_width)
@@ -413,14 +516,3 @@ class LineChart(BaseChart):
 
     # def get_frames(self):
     #     return range(len(self.df.index))
-
-    def make_animation(
-        self, filename,
-    ):
-        anim = super().make_animation(self.get_frames(), self.init_func)
-
-        extension = filename.split(".")[-1]
-        if extension == "gif":
-            anim.save(filename, fps=self.fps, writer="imagemagick")
-        else:
-            anim.save(filename, fps=self.fps)
